/*
   webresolve.c

   Copyright (C) 2000  Ulric Eriksson <ulric@siag.nu>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* The cache is implemented as a linked list, based on the assumption that
   we will usually want to get the most recently inserted items.
*/
typedef struct ip_cache {
	char *ip;
	char *host;
	struct ip_cache *next;
} ip_cache;

static ip_cache *cache = NULL;

static ip_cache *lookup_ip(char *ip)
{
	ip_cache *c;

	for (c = cache; c; c = c->next)
		if (!strcmp(ip, c->ip)) return c;
	return NULL;
}

static void reverse_lookup(char *ip, char *host)
{
	struct hostent *hp;
	struct in_addr in;
	ip_cache *c;
	if (!isdigit(*ip)) {
		strcpy(host, ip);
		return;
	}
	c = lookup_ip(ip);
	if (c) {
		strcpy(host, c->host);
		return;
	}
	inet_aton(ip, &in);
	hp = gethostbyaddr((char *)&in.s_addr, sizeof(in.s_addr), AF_INET);
	if (hp) strcpy(host, hp->h_name);
	else strcpy(host, ip);
	c = malloc(sizeof *c);
	c->host = malloc(strlen(host)+1);
	c->ip = malloc(strlen(ip)+1);
	strcpy(c->host, host);
	strcpy(c->ip, ip);
	c->next = cache;
	cache = c;
}

int main(void)
{
	char b[1024], host[1024], ip[1024], rest[1024];

	while (fgets(b, sizeof b, stdin)) {
		sscanf(b, "%s %[^\n]", ip, rest);
		reverse_lookup(ip, host);
		printf("%s %s\n", host, rest);
	}
	return 0;
}

